<?php
/**
 * MAGECONNECT
 * Connecting TYPO3 and Magento
 * 
 * Fetch Category Class
 * 
 * This class fetches category details from
 * TYPO3 and Magento
 * 
 * @author Bastian 'Aggy' Zagar
 * @version 0.1
 *
 */
require_once(t3lib_extMgm::extPath('mageconnect').'lib/class.tx_mageconnect_soap.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_registry.php');

class tx_mageconnect_category
{
	/**
	 * Soap Class
	 * @var tx_mageconnect_soap
	 */	
	private $_soap;
	
	private $cObj;
	public  $conf;
	private $controller;
	
	/**
	 * Categories
	 * @var array
	 */
	protected $_categories;
	
	function __construct()
	{

	}
	
	function __destruct()
	{
		
	}
	
	/**
	 * initRegistry
	 * Initalize registry objects
	 * 
	 * @return void
	 */
	function initRegistry()
	{
		$this->cObj 		= tx_mageconnect_registry::get('cObj');
		$this->conf 		= tx_mageconnect_registry::get('conf');
		$this->controller 	= tx_mageconnect_registry::get('controller');
		$this->controller->pi_loadLL();
	}
	
	/**
	 * _removeCategoriesWithStore
	 * Removes all categories from category cache with store view code
	 * 
	 * @param string $store Store View Code
	 * @return bool
	 */
	function _removeCategoriesWithStore($store)
	{
		$dbResource = $GLOBALS['TYPO3_DB']->exec_DELETEquery('tx_mageconnect_categories', 'store_view = \''.$store.'\' AND userdefined = \'0\'');
															 
		if ($dbResource) {
			return true;			
		}
		return false;
	}
	
	/**
	 * _connect
	 * Connect to webservice
	 */
	function _connect()
	{
		$soapUrl 	= $this->controller->_helper->getApiUrl();
		$soapUser 	= $this->controller->_helper->getApiUsername();
		$soapKey 	= $this->controller->_helper->getApiKey();
		
		$this->_soap = t3lib_div::makeInstance('tx_mageconnect_soap', 
											   $soapUrl, 
											   $soapUser, 
											   $soapKey
		);
	}
	
	/**
	 * _getConfig
	 * Return config data from a given
	 * key
	 * 
	 * @param string $key
	 * @return string
	 */
	function _getConfig($key)
	{
		return $this->conf[$key];
	}
	
	/**
	 * getCategories
	 * Get a category list
	 * 
	 * @param string $store Store View Code
	 * @param int $storagePid Pid of the category storage
	 * @return bool
	 */
	function getCategories($store = '', $storagePid)
	{
		if ($this->_categories) {
			return $this->_categories;
		}
		
		$categories = $this->getCategoriesFromDb($store, $storagePid);
		
		if ($this->getFlexformReloadCategories() || empty($categories)) {
			// Fetch categories from soap
			$this->importCategories($store, $storagePid);
		} 
		$categories = $this->getCategoriesFromDb($store, $storagePid);
		
		if (!empty($categories)) {
			$this->_categories = $categories;
		}
		return $this->_categories;
	}
	
	/**
	 * getCategoryById
	 * Gets an category array by a given id and store view
	 * 
	 * @param int $id Category Id
	 * @param string $store Store View Code
	 * @param int $storagePid Pid of the category storage
	 * @return array
	 */
	function getCategoryById($id, $store = '', $storagePid)
	{
		$categories = $this->getCategories($store, $storagePid);
		
		foreach($categories as $_category) {
			
			if ($_category['category_id'] == $id && $_category['store_view'] == $store) {
				if ($_category['parent_id']) {	
					$children = explode(',', $_category['parent_id']);
				
					$i = 0; $count = count($children);
					foreach ($children as $_childId) {
						$childCat = $this->getCategoryById($_childId, $store, $storagePid);
						
						if ($i == 0) {
							$childCat['class'] = 'first sub';
						}
						if ($i+1 == $count) {
							$childCat['class'] = 'last sub';
						}
						
						$_category['children'][] = $childCat;
						
						$i++;
					}
				} 
				return $_category;
			}
		}
		return;
	}
	
	/**
	 * buildTree
	 * Build a tree from a given array
	 * 
	 * @param array $elements Array with Elements
	 * @param int $parentId Parent Id
	 * @return array
	 */
	function buildTree(array &$elements, $parentId = 0, $level = 0)
	{
		$branch = array();

		foreach ($elements as $key=>$_element) {
			$_element['level'] = $level;
			
			if ($level == 0) {
				$_element['level-top'] = true;
			} else {
				$_element['level-top'] = false;
			}
			
			if ($_element['parent_id'] == $parentId) {
				$children = $this->buildTree($elements, $_element['category_id'], $level+1);
				if ($children) {
					$_element['children'] = $children;
					$_element['has_children'] = true;
				} else {
					$_element['has_children'] = false;
				}
				$branch[] = $_element;
				
			}
		}
		return $branch;
	}

	/**
	 * getCategoryTree
	 * Get an tree array with the category structure 
	 * 
	 * @param string $store Store View Code
	 * @param string $storagePid Pid of the category storage
	 * @return array
	 */
	function getCategoryTree($store = '', $storagePid)
	{
		$categories = $this->getCategories($store, $storagePid);
		$tree = array();
		
		if (!empty($categories)) {
			
			$root = array(); $childs = array(); $flat = array(); 
 			foreach ($categories as $_cat) {
 				$childs[] = $_cat;
				$root[$_cat['parent_id']][$_cat['category_id']] = $_cat['name'];
 			}
			
			$rootId = key($root);
			$tree = $this->buildTree($childs, $rootId);
		}
		return $tree;
	}
	
	/**
	 * getCategoriesFromDb
	 * Gets categories from database
	 * 
	 * @param string $store Store View Code
	 * @param string $storagePid Pid of the category storage
	 * @return array
	 */
	function getCategoriesFromDb($store = '', $storagePid)
	{
		$categories = array();
		
		$dbResource = $GLOBALS['TYPO3_DB']->exec_SELECTquery('*', 							// SELECT
															 'tx_mageconnect_categories', 	// FROM
															 '  store_view = \''.$store.'\'
															 	AND hidden = \'0\' 
															 	AND deleted = \'0\'
															 	AND pid = \''.$storagePid.'\'',	// WHERE
															 '',							// GROUP BY
															 'sorting,uid',					// ORDER BY
															 ''								// LIMIT
															);
		if ($dbResource) {
			
			while($_category = $GLOBALS['TYPO3_DB']->sql_fetch_assoc($dbResource)) {
				$categories[] = $_category;
			}
			$GLOBALS['TYPO3_DB']->sql_free_result($dbResource);
			
		}
		return $categories;
	}
	
	/**
	 * importCategories
	 * Import a category list from the webservice
	 * 
	 * @param string $store Store Code
	 * @param int $storagePid Pid of the category storage
	 * @return array
	 */
	function importCategories($store = '', $storagePid)
	{
		$this->_connect();
		$categories = $this->_soap->getCategoryList($store);	
		
		$status = null;
	
		if ($categories) {
		
			// Truncate category cache tx_mageconnect_categories
			$this->_removeCategoriesWithStore($store);
			
			$sorting = 2048;
			foreach ($categories as $_category) {
				
				$category = array(
					'pid'					=> $storagePid,
					'tstamp'				=> time(),
					'crdate'				=> time(),
					'sorting'				=> $sorting,
					'category_id'			=> $_category['category_id'],
					'parent_id'				=> $_category['parent'],
					'name'					=> $_category['name'],
					'description'			=> $_category['description'],
					'page_title'			=> $_category['page_title'],
					'image'					=> $_category['thumbnail'],
					'url'					=> $_category['url'],
					'url_key'				=> $_category['url_key'],
					'url_path'				=> $_category['url_path'],
					'product_count'			=> $_category['product_count'],
					'product_ids'			=> $_category['product_ids'],
					'store_view'			=> $store,
					'attributes'			=> serialize($_category['attributes'])
				);
				$result = $this->insertCategory($category);
				$sorting += 128;
				
				if ($result !== true) {
					$status = false;
				}
			}
		}
		return ($status === null)?true:false;
	}
	
	/**
	 * insertCategory
	 * Inserts an category to database
	 * 
	 * @param array $categoryData
	 * @return bool
	 */
	public function insertCategory($categoryData)
	{
		try {
			$insertQuery = $GLOBALS['TYPO3_DB']->exec_INSERTquery('tx_mageconnect_categories', $categoryData);
		
		} catch (Exception $e) { 
			return false;
		}
		return true;
	}

	/**
	 * updateCategory
	 * Updates an category in the database
	 * 
	 * @param int $categoryId
	 * @param array $categoryData
	 * @param string $store Store View Code
	 * @return bool
	 */
	public function updateCategory($categoryId, $categoryData, $store = '')
	{
		try {
			$updateQuery = $GLOBALS['TYPO3_DB']->exec_UPDATEquery('tx_mageconnect_categories', 'category_id=\''. $categoryId . '\' AND store_view = \''. $store . '\'', $categoryData);
		} catch (Exception $e) { 
			return false;
		}
		return true;
	}	
	
	/**
	 * _getFlexformConfig
	 * Get the flexform config by typename and fieldname
	 * 
	 * @param string $type
	 * @param string $field
	 * @return string
	 */
	function _getFlexformConfig($field, $type)
	{
		if (method_exists($this, 'pi_getFFValue')) {
			return $this->pi_getFFValue($this->cObj->data['pi_flexform'], $field, $type);
		} 
		return '';
	}	
	
	/**
	 * getFlexformStoreView
	 * Gets the flexform storeview setting
	 * 
	 * @return string
	 */
	function getFlexformStoreView()
	{
		$storeview_code = $this->_getFlexformConfig('store_view_code', 'general_setting');
		return (string)$storeview_code;
	}
	
	/**
	 * getFlexformReloadCategories
	 * Gets the flexform setting for reloading categories
	 * 
	 * @return bool
	 */
	function getFlexformReloadCategories()
	{
		$reload = $this->_getFlexformConfig('always_load_from_soap', 'general_setting');
		return (bool)$reload;
	}
	
}
	