<?php
/**
 * MAGECONNECT
 * Connecting TYPO3 and Magento
 * 
 * Crypt Class
 * 
 * Encrypt and Decrypt
 * 
 * @author Bastian 'Aggy' Zagar
 * @version 0.1
 *
 */
 
class tx_mageconnect_crypt
{
	/**
	 * Encryption Key
	 * @var string
	 */	
	private $_encKey;
	
	/**
	 * __construct
	 * Constructor
	 */
	function __construct($encKey)
	{
		$this->_encKey = $encKey;
	}
	
	/**
	 * __destruct
	 * Destructor
	 */
	function __destruct()
	{
		$this->_reset();
		unset($this->_encKey);
	}
	
	/**
	 * _reset
	 * Resets the encryption key
	 * 
	 * @return void
	 */
	function _reset()
	{
		$this->_encKey = NULL;
	}
	
	/**
	 * _getKey
	 * Get encryption key
	 * 
	 * @return string
	 */
	protected function _getKey()
	{
		return $this->_encKey;
	}
	
	/**
	 * getCrypted
	 * Get an encrypted string
	 * 
	 * @param string $string String to encrypt
	 * @return string Encrypted String
	 */
	public function getCrypted($string)
	{
		return $this->_encrypt($string);
	}
	
	/**
	 * getDecrypted
	 * Get a decrypted string from a given encrypted string
	 * 
	 * @param string $string Encrypted String
	 * @return string Decrypted String
	 */
	public function getDecrypted($string)
	{
		return $this->_decrypt($string);
	}
	
	/**
	 * _encrypt
	 * Encrypts a string
	 * 
	 * @param string $string String to encrypt
	 * @return string
	 */
	private function _encrypt($string)
	{
		$string = trim($string);
		$key = $this->_getKey();
		
		$iv = mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND);
		$passcrypt = mcrypt_encrypt(MCRYPT_RIJNDAEL_256, $key, $string, MCRYPT_MODE_ECB, $iv);
		$encode = base64_encode($passcrypt);
		return $encode;
	}	
	
	/**
	 * _decrypt
	 * Decrypts a string
	 * 
	 * @param string $encString String to decrypt
	 * @return string
	 */
	private function _decrypt($encString)
	{
		$string = trim($string);
		$key = $this->_getKey();
		
		$decoded = base64_decode($encString);
		$iv = mcrypt_create_iv(mcrypt_get_iv_size(MCRYPT_RIJNDAEL_256, MCRYPT_MODE_ECB), MCRYPT_RAND);
		$decrypted = mcrypt_decrypt(MCRYPT_RIJNDAEL_256, $key, $decoded, MCRYPT_MODE_ECB, $iv);
		return $decrypted;
	}
}


if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_crypt.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_crypt.php']);
}

