<?php
class tx_mageconnect_helper
{
	const REDIRECT_METHOD_TYPO3 	= 1;
	const REDIRECT_METHOD_HEADER 	= 2;
	
	const EXT_NAME = 'mageconnect';
	
	/**
	 * _getParam
	 * Gets an parameter and decrypts it for
	 * usage
	 * 
	 * @param string Parameter
	 * @return string
	 * 
	 */	
	function _getParam($parameter = 'data')
	{
		if (t3lib_div::_GP($parameter)) {
			return t3lib_div::_GP($parameter);
		}
		return;	
	}
	
	/**
	 * getModuleConfig
	 * Gets Module Config for MAGEconnect
	 * 
	 * @param string $key
	 * @return string|bool
	 */
	function _getModuleConfig($key = '')
	{
		$modConfigArray = unserialize( $GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf'][self::EXT_NAME]);
		if ($modConfigArray[$key]) {
			return $modConfigArray[$key];
		}
		return false;
	}
	
	/**
	 * isLoggedIn
	 * Determine if the customer is logged in
	 * and return bool
	 * 
	 * @return bool
	 */
	function isLoggedIn()
	{
		$uid = $GLOBALS['TSFE']->fe_user->user['uid'];
		
		if ($uid) {
			return true;
		}
		return false;
	}
	
	/**
	 * getFEUserDetail
	 * Get a frontend user detail
	 * 
	 * @param string $detail Detail Name
	 * @return string
	 */
	function getFEUserDetail($detail)
	{
		if ($this->isLoggedIn()) {
			if (array_key_exists($detail, $GLOBALS['TSFE']->fe_user->user)) {
				return $GLOBALS['TSFE']->fe_user->user[$detail];
			}
		}	
		return;
	}
	
	/**
	 * getCustomerUid
	 * Gets customers uid
	 * 
	 * @return int|null
	 */
	function getCustomerUid()
	{
		if ($this->isLoggedIn()) {
			$uid = $this->getFEUserDetail('uid');
		
			if ($uid) {
				return $uid;
			}
		}
		return false;
	}

	/**
	 * getTYPO3connectUrl
	 * Gets the Url for the TYPO3 connect Module that
	 * is located in the magento store
	 * 
	 * @return string
	 */
	function getTYPO3connectUrl()
	{
		return $this->_getModuleConfig('magento_url') . 'customer/account/loginConnect/';
	}	
	
	/**
	 * getForgotPasswordUrl
	 * Gets the Url for the ajax forgot password controller
	 * 
	 * @return string
	 */
	function getForgotPasswordUrl()
	{
		return $this->_getModuleConfig('magento_url') . 'customer/account/forgotPasswordAjax/';
	}	
	
	/**
	 * getCustomerAccountUrl
	 * Gets magento customer/account URL
	 * 
	 * @return string
	 */
	function getCustomerAccountUrl()
	{
		return $this->_getModuleConfig('magento_url') . 'customer/account/';
	}
	
	/**
	 * getDefaultImportAddressArrayKey
	 * Gets the array key for getting the desired
	 * address to import
	 * 
	 * @return string (either billing or shipping)
	 */
	function getDefaultImportAddressArrayKey()
	{
		$configStr = $this->_getModuleConfig('import_address');
		
		if ($configStr != 'Import Default Billing Address') {
			return 'shipping';
		}
		return 'billing';
	}
		
	/**
	 * getMagentoUrl
	 * Gets the magento url from the module config
	 * 
	 * @return string
	 */
	function getMagentoUrl()
	{
		return $this->_getModuleConfig('magento_url');
	}
	
	/**
	 * getMagentoBaseUrl
	 * Cleans the magento url of index.php/
	 * and returns the url
	 * 
	 * @return string
	 */
	function getMagentoBaseUrl()
	{
		$url = $this->getMagentoUrl();
		$path = str_replace('/index.php', '', $url);
		$path = rtrim($path, '/').'/';
		
		return $path;
	}
	
	/**
	 * getApiUrl
	 * Gets the soap api url for magento soap requests
	 * 
	 * @return string
	 */
	function getApiUrl()
	{
		$baseUrl = $this->getMagentoUrl();
		return rtrim($baseUrl, '/') . '/api/v2_soap?wsdl=1';		
	}
	
	/**
	 * getApiUsername
	 * Get the api username from the module configuration
	 * 
	 * @return string
	 */
	function getApiUsername()
	{
		return $this->_getModuleConfig('magento_api_username');
	}
	
	/**
	 * getApiKey
	 * Get the api key from the module configuration
	 * 
	 * @return string
	 */	
	function getApiKey()
	{
		return $this->_getModuleConfig('magento_api_key');
	}	
	
	/**
	 * getCacheLifetime
	 * Gets the cache lifetime from the module configuration
	 * 
	 * @return int
	 */
	function getCacheLifetime()
	{
		return (int)$this->_getModuleConfig('cache_lifetime');		
	}
		
	/**
	 * getScriptUrl
	 * Gets the script url for this login form
	 * 
	 * @return string
	 */
	function getScriptUrl()
	{
		return t3lib_div::getIndpEnv(TYPO3_SITE_URL) . 'index.php?id=' . $GLOBALS['TSFE']->id;
	}

	/**
	 * isInDebugMode
	 * Determines if extension is currently 
	 * in debug mode
	 * 
	 * @return bool
	 */
	function isInDebugMode()
	{
		return $this->_getModuleConfig('debug_mode');
	}
	
	/**
	 * getRedirectMethod
	 * Gets the configuration of the redirect method
	 * 
	 * @return string the method
	 */
	function getRedirectMethod()
	{
		$config = $this->_getModuleConfig('redirect_method');
		
		switch ($config) {
			case 't3lib_utility_Http::redirect':
				return self::REDIRECT_METHOD_TYPO3;	
			case 'header(location:)':
			default:
				return self::REDIRECT_METHOD_HEADER;

		}
	}

	/**
	 * getEncryptDecryptKey
	 * Gets the encryption/decryption key from
	 * the module setting
	 * 
	 * @return string
	 */
	function getEncryptDecryptKey()
	{
		return $this->_getModuleConfig('encrypt_decrypt_key');
	}

	/**
	 * getAttributeValue
	 * Gets an attribute value from an
	 * serialized string
	 * 
	 * @param string $attributeCode
	 * @param string $serializedStr
	 * @return string
	 */	
	function getAttributeValue($attributeCode, $serializedStr)
	{
		$_attributes = unserialize($serializedStr);
		
		foreach ($_attributes as $_code=>$_value) {
			if ($_code === $attributeCode) {
				return $_value;
			}
		}
		return '';
	}
	
	/**
	 * getMarkers
	 * Generate an marker array by an given data
	 * array
	 * 
	 * @param 	array 	$data
	 * @return	array
	 */
	function getMarkers($data = array(), $prefix = null) 
	{
		$markers = array();
			
		if ($prefix) {
			$prefix = $prefix . '_';
		}
			
		foreach ($data as $_field=>$_value) {
			if (!is_array($_value)) {
				$_marker = '###' . $prefix . strtoupper($_field) . '###';	
				$markers[$_marker] = $_value;
			}
		}
		
		return $markers;
	}
	
	/**
	 * displayError
	 * Displays an errormessage
	 *
	 * @return	string		the error message
	 */
	function displayError($errormsg) 
	{
		$flashMessage = t3lib_div::makeInstance(
			't3lib_FlashMessage',
			$errormsg,
			'tx_mageconnect Error',
			t3lib_FlashMessage::ERROR
		);
		return $flashMessage->render();
	}
		
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_helper.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/lib/class.tx_mageconnect_helper.php']);
}
