<?php
require_once(PATH_tslib.'class.tslib_pibase.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_helper.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_soap.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_cache.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_registry.php');

require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_category.php');

class tx_mageconnect_flexform extends tslib_pibase
{
	const cacheId = 'flexformList';
	
	protected $_products = array();
	protected $_categories = array();
	protected $_soap;
	protected $_helper;
	protected $_cache;
	
	function __construct()
	{
		$this->_helper 	= t3lib_div::makeInstance('tx_mageconnect_helper');
	}
	
	function _connect()
	{
		$soapUrl 	= $this->_helper->getApiUrl();
		$soapUser 	= $this->_helper->getApiUsername();
		$soapKey 	= $this->_helper->getApiKey();
		
		$this->_soap = t3lib_div::makeInstance('tx_mageconnect_soap', 
											   $soapUrl, 
											   $soapUser, 
											   $soapKey
		);
	}
	
	/**
	 * initCache
	 * initialize Caching Instance
	 */
	function initCache()
	{
		$this->_cache = t3lib_div::makeInstance('tx_mageconnect_cache');
	}

	/**
	 * _getFlexformConfig
	 * Get the flexform config by typename and fieldname
	 * 
	 * @param string $type
	 * @param string $field
	 * @return string
	 */
	function _getFlexformConfig($field, $type)
	{
		$flexform = t3lib_div::xml2array($this->conf['row']['pi_flexform']);
		$languageKey = 'lDEF';
		
		if (array_key_exists('data', $flexform) && !empty($flexform['data']) && $flexform['data']) {
			return $flexform['data'][$type][$languageKey][$field]['vDEF'];
		}
		return '';
	}


	/**
	 * populateProductList
	 * Populates the product option list in flexform
	 * 
	 * @param array $config
	 * @param array $transferdata
	 * @return array
	 */
	function populateProductList($config, $transferdata)
	{
		$this->conf = $config;
		
		$this->initCache();
		
		// Cache Lifetime
		$lifetime = $this->_helper->getCacheLifetime();
		
		// Store View Code from Flexform
		$store_view_code = $this->_getFlexformConfig('store_view_code', 'product_setting');
		
		// Cache ID for Store View Code
		$cacheId = self::cacheId . '_products_' . $store_view_code;
		
		$optionList = array();
		if (FALSE === ($optionList = $this->_cache->get($cacheId))) {
			
		
			if (empty($this->_products)) {
					
				try {
					$this->_connect();
					$this->_products = $this->_soap->getProductsList($store_view_code);
				} catch (Exception $e) {
					die('Could not fetch products list: ' . $e);
				}
				
			} 
			
			foreach ($this->_products as $_product) {
					
				$productId 		= $_product->product_id;
				$productSku		= $_product->sku;
				$productName 	= $_product->name;
				
				$optionStr = '['.$productSku.'] ' . $productName;
				
				$optionList[] = array(0 => $optionStr, 1 => $productId);
			}
			$this->_cache->set( $cacheId, $optionList, array(), $lifetime );
		
		} 
		$config['items'] = array_merge($config['items'],$optionList);
		return $config;	
	}

	function getCategoriesPid()
	{
		$flexform = $this->_getFlexformConfig('categoriesPid','general_setting');
		
		$intString = '';
		if (strstr($flexform,'pages_')) {
			$intString = str_replace('pages_','',$flexform);
			$intString = substr($intString,0,strpos($intString,'|'));
			$intString = intval($intString);
			
		} else {
			$intString = intval($flexform);
		}
		return $intString;
	}

	/**
	 * populateCategoryList
	 * Populates the category option list in flexform
	 * 
	 * @param array $config
	 * @param array $transferdata
	 * @return array
	 */
	function populateCategoryList($config, $transferdata)
	{
		$this->conf = $config;
		
		$this->initCache();
		
		// Cache Lifetime
		$lifetime = $this->_helper->getCacheLifetime();
		
		// Cache ID for Store View Code
		$cacheId = self::cacheId . '_categories_' . $store_view_code;
		
		// Register Objects
		tx_mageconnect_registry::set($config, 'conf');
		//tx_mageconnect_registry::set($this->cObj, 'cObj');
		tx_mageconnect_registry::set($this, 'controller');
		
		$categoryClass = t3lib_div::makeInstance('tx_mageconnect_category');
		
		// Store View Code from Flexform
		$store_view_code = $this->_getFlexformConfig('store_view_code', 'product_setting');
			
			
		$optionList = array();
		if (FALSE === ($optionList = $this->_cache->get($cacheId))) {
			if (empty($this->_categories)) {
				
				// We try to find categories in database first				
				$this->_categories = $categoryClass->getCategoriesFromDb($store_view_code, $this->getCategoriesPid());
				
				// No categories in database :|, SOAP will get 'em
				if (!$this->_categories || empty($this->_categories)) {
					try {
						$this->_connect();
						$this->_categories = $this->_soap->getCategoryList($store_view_code);
					} catch (Exception $e) {
						die('Could not fetch categories list: ' . $e);
					}
				}
			}
			
			foreach ($this->_categories as $_category) {
					
				$categoryId 		= $_category['category_id'];
				$categoryName		= $_category['name'];
				$products			= explode(',', $_category['product_ids']);
					
				$optionStr = '['.$categoryId.'] ' . ' ' . $categoryName .' '. '(Count: '.count($products).')';
				
				$optionList[] = array(0 => $optionStr, 1 => $categoryId);
			}
			$this->_cache->set( $cacheId, $optionList, array(), $lifetime );
		}
		$config['items'] = array_merge($config['items'],$optionList);
		return $config;	
	}

}

