<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2011 Bastian Zagar (magedeveloper.de) <kontakt@magedeveloper.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 */
require_once(PATH_tslib.'class.tslib_pibase.php');

require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_category.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_helper.php');
require_once(t3lib_extMgm::extPath('mageconnect') . 'lib/class.tx_mageconnect_registry.php');

/**
 * Plugin 'MAGEconnect Category Tree Display' for the 'mageconnect' extension.
 *
 * @author	Bastian Zagar (magedeveloper.de) <kontakt@magedeveloper.de>
 * @package	TYPO3
 * @subpackage tx_mageconnect
 */
class tx_mageconnect_pi4 extends tslib_pibase {
	
	var $prefixId      = 'tx_mageconnect_pi4';		// Same as class name
	var $scriptRelPath = 'pi4/class.tx_mageconnect_pi4.php';	// Path to this script relative to the extension dir.
	var $extKey        = 'mageconnect';	// The extension key.
	var $pi_checkCHash = true;
	
	var $_helper;
	var $_pid;
	var $conf;
	
	var $_category;
	
	var $_debugData = array();	
	var $_templateHtml;
	
	/**
	 * __construct
	 * Constructor
	 */
	function __construct()
	{
		parent::__construct();
		$this->_helper 		= t3lib_div::makeInstance('tx_mageconnect_helper'); 
	}
	
	/**
	 * _getFlexformConfig
	 * Get the flexform config by typename and fieldname
	 * 
	 * @param string $type
	 * @param string $field
	 * @return string
	 */
	function _getFlexformConfig($field, $type)
	{
		return $this->pi_getFFValue($this->cObj->data['pi_flexform'], $field, $type); 
	}	
	
	
	
	/**
	 * getStoreViewCode
	 * Get the store view code from the flexform or config setting
	 * 
	 * @return string
	 */
	function getStoreViewCode()
	{
		$configStore = $this->_getFlexformconfig('store_view_code', 'general_setting');
		
		if (!$configStore) {
			return $this->_getConfig('storeViewCode');
		}
		return $configStore;
	}
	
	/**
	 * _getConfig
	 * Return config data from a given
	 * key
	 * 
	 * @param string $key
	 * @return string
	 */
	function _getConfig($key)
	{
		return $this->conf[$key];
	}	
	
	/**
	 * _getSubpartHtml
	 * Gets the html subparts for category children
	 * 
	 * @param array $children Children Categories
	 * @return string
	 */
	function _getSubpartHtml($children = array(), $level = 0)
	{
		// Load Template File
		$div 			= $this->cObj->getSubpart($this->_templateHtml, '###CATEGORY_LIST###');
		$ul				= $this->cObj->getSubpart($this->_templateHtml, '###UL###');
		$li				= $this->cObj->getSubpart($this->_templateHtml, '###LI###');
		$li_inner		= $this->cObj->getSubpart($this->_templateHtml, '###LI_INNER###');
		
		$subHtml = ''; $content = '';
		foreach ($children as $_childCat) {
			
			$innerContent = ''; $childContent = '';
			
			// MARKERS
			$css = array(
				'level' 		=> 'level'.$_childCat['level'],
				'top-level' 	=> ($_childCat['level-top'])?'level-top':'',
				'parent'		=> ($_childCat['has_children'])?'parent':''
			);
			
			$attributes = array();
			if ($_childCat['attributes']) {
				$attributes = unserialize($_childCat['attributes']);
				unset($_childCat['attributes']);
			}
			$data = array_merge($attributes, $_childCat, $css);
			$markers = $this->_helper->getMarkers($data);
			
			$innerContent = $this->cObj->substituteMarkerArray($li_inner, $markers);	
			$content = $this->cObj->substituteMarkerArray($li, $markers);
			
			
			if (array_key_exists('children', $_childCat)) {
				$innerContent .= $this->_getSubpartHtml($_childCat['children'], $level+1);
			}
			$subHtml .= $this->cObj->substituteSubpart($content, '###LI_INNER###', $innerContent);	
			
		}
		$ulWithMarkers = $this->cObj->substituteMarkerArray($ul, array('###LEVEL###'=>'level'.$level));	
		return $this->cObj->substituteSubpart($ulWithMarkers, '###LI###', $subHtml);;
	}
	
	/**
	 * _getCategoryTreeHtml
	 * Displays the category tree
	 * template
	 * 
	 * @return string
	 */
	function _getCategoryTreeHtml()
	{
		$tree = $this->getCategoryTree();
		
		// Load Template File
		$div 			= $this->cObj->getSubpart($this->_templateHtml, '###CATEGORY_LIST###');
		$ul				= $this->cObj->getSubpart($this->_templateHtml, '###UL###');
		$li				= $this->cObj->getSubpart($this->_templateHtml, '###LI###');
		$li_inner		= $this->cObj->getSubpart($this->_templateHtml, '###LI_INNER###');
		
		$subpartContent = ''; $content = ''; $markers = array();
		foreach ($tree as $_key=>$_cat) {
			
			$innerContent = '';
			
			// MARKERS
			$css = array(
				'level' 		=> 'level'.$_cat['level'],
				'top-level' 	=> ($_cat['level-top'])?'level-top':'',
				'parent'		=> ($_cat['has_children'])?'parent':''
			);
			
			$attributes = array();
			if ($_childCat['attributes']) {
				$attributes = unserialize($_childCat['attributes']);
				unset($_childCat['attributes']);
			}
			
			$data = array_merge($attributes, $_cat, $css);
			$markers = $this->_helper->getMarkers($data);

			$innerContent .= $this->cObj->substituteMarkerArray($li_inner, $markers);
					
			if (array_key_exists('children',$_cat)) {
				$innerContent .= $this->_getSubpartHtml($_cat['children']);
			} 
			
			$subpartContent = $this->cObj->substituteMarkerArray($li, $markers);		
			$content .= $this->cObj->substituteSubpart($subpartContent, '###LI_INNER###', $innerContent);	
		}
		$ulWithMarkers = $this->cObj->substituteMarkerArray($ul, array('###LEVEL###'=>'root'));	
		$content = $this->cObj->substituteSubpart($ulWithMarkers, '###LI###', $content);	
		return $this->cObj->substituteSubpart($div, '###UL###', $content);	
	}
	
	/**
	 * getStoragePid
	 * Gets the storage pid for the category data either from the config setting
	 * or flexform configuration
	 * 
	 * @return int Storage PID
	 */
	function getStoragePid()
	{
		$configPid = $this->_getConfig('storagePid');
		if (!$configPid) {
			return $this->_getFlexformConfig('categoriesPid', 'general_setting');
		}
		return $configPid;
	}
	
	/**
	 * getCategoryTree
	 * Fetch category tree
	 * 
	 * @return array
	 */
	function getCategoryTree()
	{
		$store = $this->getStoreViewCode();
		$this->_debugData['store_view_code'] = $store;
		$storagePid = $this->getStoragePid();
		$this->_debugData['storage_pid'] = $storagePid;
		
		$tree = $this->_category->getCategoryTree($store, $storagePid);
		
		if ($tree) {
			$this->_debugData['category_data'] = $tree;
			return $tree;
		}
		return;
	}
		
	/**
	 * The main method of the PlugIn
	 *
	 * @param	string		$content: The PlugIn content
	 * @param	array		$conf: The PlugIn configuration
	 * @return	The content that is displayed on the website
	 */
	function main($content, $conf) 
	{
		$this->pi_initPIflexForm();    // Initialize the FlexForms array 
		
		$this->conf = $conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		
		// Template
		$this->_templateHtml = $this->cObj->fileResource( $this->_getConfig('templateCategoryTree') );
		
		// Register Objects
		tx_mageconnect_registry::set($conf, 'conf');
		tx_mageconnect_registry::set($this->cObj, 'cObj');
		tx_mageconnect_registry::set($this, 'controller');
		
		// Initialize Products Class
		$this->_category = t3lib_div::makeInstance('tx_mageconnect_category');
		$this->_category->initRegistry();
		
		$content = '';
		$content .= $this->_getCategoryTreeHtml();		
		
		// Show Debug Information
		if ($this->_helper->isInDebugMode() == true) {
			t3lib_utility_Debug::debug($this->_debugData);
		}
		
		return $this->pi_wrapInBaseClass($content);
	}
}

if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/pi4/class.tx_mageconnect_pi4.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/mageconnect/pi4/class.tx_mageconnect_pi4.php']);
}


?>